import 'package:flutter_test/flutter_test.dart';

import 'package:asis/list.dart';

void main() {
  group('List<T> extension', () {
    var list = [1, 2, 3];
    group('Strict', () {
      test('Transformation with as<T>', () {
        expect(list.as<String>((e) => e.toString()), ['1', '2', '3']);

        expect(list.as<String>((e) => null), [null, null, null]);

        expect(
          list.as<Map<String, int>>((e) => {e.toString(): e * e}),
          [
            {'1': 1},
            {'2': 4},
            {'3': 9}
          ],
        );
      });

      test('Transformation with the result of as<T>', () {
        expect(
          list.as<String>((e) => e.toString()).as<String>((e) => e + e),
          ['11', '22', '33'],
        );

        expect(list.as<String>((e) => null).as<int>((e) => 1), [1, 1, 1]);

        expect(
          list
              .as<Map<String, int>>((e) => {e.toString(): e * e})
              .as<String>((e) => e.toString()),
          ['{1: 1}', '{2: 4}', '{3: 9}'],
        );
      });
    });
    group('Automatic', () {
      test('Transformation with as<T>', () {
        expect(list.as((e) => e.toString()), ['1', '2', '3']);

        expect(list.as((e) => null), [null, null, null]);

        expect(
          list.as((e) => {e.toString(): e * e}),
          [
            {'1': 1},
            {'2': 4},
            {'3': 9}
          ],
        );
      });

      test('Transformation with the result of as<T>', () {
        expect(
          list.as((e) => e.toString()).as((e) => e + e),
          ['11', '22', '33'],
        );

        expect(list.as((e) => null).as((e) => 1), [1, 1, 1]);

        expect(
          list.as((e) => {e.toString(): e * e}).as((e) => e.toString()),
          ['{1: 1}', '{2: 4}', '{3: 9}'],
        );
      });
    });
  });
}
